#!/usr/bin/perl

# Args: strain names; first is main one, rest are ones to compare against.
# Run this script in a directory where files p?/*/sbw.txt exist. They
# should have been created by snpsByWindows.pl. It will create files
# plots/STR/CHR.png, where STR is the main strain and CHR is a chromosome
# name (corresponding to the "*" in the sbw.txt file pattern).

# Example, in a "cere" directory:
#
# plotStrainSimilarities.pl SK1 REF NCYC110 RM11_1A 

use GD::Simple; # you may need to grab this from CPAN
use Utility; # DMC's utility code, e.g. in same directory as this script
use File::Basename;
use strict;

my @COLOURS = qw(black red green blue orange purple brown);

unless (@ARGV) {
    die("Usage: $0 mainStrain compStrain1 compStrain2 ...");
}

my ($str0,@strains) = @ARGV;
my %strIdx;
foreach my $k (0 .. $#strains) {
    $strIdx{$strains[$k]} = $k;
}

# sbw.txt files are created by snpsByWindows.pl

my @sbwFiles = glob("p?/chr??/sbw.txt");
unless (@sbwFiles) {
    die("$0: cannot find p?/chr??/sbw.txt");
}

foreach my $f (@sbwFiles) {
    my $chr = basename(dirname($f));
    my $g = "plots/$str0/$chr.png";
    report("creating $g");
    my $fh = openToRead($f);
    my @sim;
    my $bMax;
    my $simMax;
    while (<$fh>) {
	chomp;
	my ($s1,$s2,undef,$block,undef,undef,undef,$n,undef,$d) = split;
	next if ($d < 50); # uncertainty
	my $idx = $s1 eq $str0 ? $strIdx{$s2} : $s2 eq $str0 ? $strIdx{$s1} : undef;
	if (defined $idx) {
	    #my $dib = $n*10000/$d;
	    my $sib = $d/($n || 1);
	    $sim[$block][$idx] = $sib;
	    $bMax = max($block,$bMax);
	    $simMax = max($sib,$simMax);
	}
    }
    my $spaceX = 100;
    my $dimX = 1000+$spaceX;
    my $sx = 1000/($bMax+3);
    my $dimY = 500;
    my $sy = $dimY/($simMax*scalar(@strains));
    my $im = new GD::Simple($dimX,$dimY+100);
    $im->fgcolor('gray');
    for (my $b=0; $b<=$bMax; $b+=10) {
	my $xx = $spaceX+$sx*$b;
	$im->moveTo($xx,0);
	$im->lineTo($xx,$dimY);
    }
    foreach my $idx (0 .. $#strains) {
	for (my $y=0; $y<=$simMax; $y+=1000) {
	    my $yy = $dimY-$sy*($y+$idx*$simMax);
	    $im->moveTo($spaceX,$yy);
	    $im->lineTo($dimX,$yy);
	}
    }
    $im->fgcolor('black');
    $im->moveTo(10+$spaceX,$dimY+30);
    my $legend = "Similarity to $str0 in $chr";
    $legend .= "; vertical bars every 100k";
    $legend .= "; horizontal bars every 1000";
    $legend .= sprintf("; peak is %d",int($simMax));
    $im->string($legend);
    foreach my $idx (0 .. $#sim) {
	$im->fgcolor($COLOURS[$idx%scalar(@COLOURS)]);
	$im->moveTo(10,$dimY-$sy*$simMax*$idx);
	$im->string($strains[$idx]);
    }
    foreach my $block (0 .. $bMax) {
	my $pA = $sim[$block];
	if (defined $pA) {
	    foreach my $idx (0 .. $#$pA) {
		$im->fgcolor($COLOURS[$idx%scalar(@COLOURS)]);
		$im->bgcolor($COLOURS[$idx%scalar(@COLOURS)]);
		if (defined $pA->[$idx]) {
		    my $xx = $spaceX+$sx*$block;
		    my $yy2 = $dimY-$sy*($simMax*$idx);
		    my $yy1 = $yy2-$sy*$pA->[$idx];
		    $im->rectangle($xx,$yy1,$xx+$sx,$yy2);
		}
	    }
	}
    }
    my $gh = openToWrite($g);
    binmode $gh;
    print $gh $im->png;
    report();
}

	    
